"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const cli_ux_1 = require("cli-ux");
const fs = require("fs");
const path = require("path");
const SymbolTree = require("symbol-tree");
const base_1 = require("../base");
const fetchChildren_1 = require("../utils/data/fetchChildren");
const fetchCompleted_1 = require("../utils/data/fetchCompleted");
const fetchInitiatives_1 = require("../utils/data/fetchInitiatives");
const crunchRoadmap_1 = require("../utils/initiatives/crunchRoadmap");
const getEmptyCalendar_1 = require("../utils/initiatives/getEmptyCalendar");
const getEmptyRoadmap_1 = require("../utils/initiatives/getEmptyRoadmap");
const prepareInitiativesData_1 = require("../utils/initiatives/prepareInitiativesData");
const prepareOrphanIssues_1 = require("../utils/initiatives/prepareOrphanIssues");
const teamVelocityFromCache_1 = require("../utils/roadmap/teamVelocityFromCache");
const treeUtils_1 = require("../utils/misc/treeUtils");
const optimizePayload_1 = require("../utils/initiatives/optimizePayload");
class Roadmap extends base_1.default {
    async run() {
        const { flags } = this.parse(Roadmap);
        // eslint-disable-next-line
        let { type, cache } = flags;
        const userConfig = this.userConfig;
        const cacheDir = this.config.configDir + '/cache/';
        if (cache) {
            this.log('==================================================================================');
            this.log('Will be fetching data from cache. NO CALLS WILL BE MADE TO JIRA TO REFRESH DATA ');
            this.log('==================================================================================');
        }
        // Creates an array of all closed issues across all teams
        // this is needed to identify which issues have been closed but are not assigned to an initiative
        // This also filters out to only issues listed in the roadmap.teams section, to avoid catch-all rules and issues from being counted twice
        let closedIssues = [];
        for (const team of userConfig.teams.filter((t) => userConfig.roadmap.teams.find((teamName) => t.name === teamName) !== undefined)) {
            const teamIssues = await fetchCompleted_1.default(userConfig, this.config.configDir + '/cache/', team.name);
            // Merge the two arrays but filter out tickets already present (i.e that could be obtained by different queries)
            closedIssues = [
                ...closedIssues,
                ...teamIssues.filter((tIssue) => closedIssues.find((cIssue) => cIssue.key === tIssue.key) ===
                    undefined),
            ];
        }
        this.log('Number of issues closed in the period: ' + closedIssues.length);
        // Build an empty weekly calendar, from the first closed issue to the last closed issue
        const emptyCalendar = getEmptyCalendar_1.default(closedIssues, userConfig);
        const velocityTeamCache = await teamVelocityFromCache_1.default(userConfig, cacheDir);
        this.log('Fetched velocity metrics for ' + velocityTeamCache.length + ' teams');
        // Fetch all initiatives and its children
        const initiativesIssues = await fetchInitiatives_1.default(userConfig, cacheDir, cache);
        this.log('Number of initiatives fetched: ' + initiativesIssues.length);
        // Fetch all of the initiatives children and build a tree
        const issuesTree = new SymbolTree();
        const treeRoot = {};
        for (const initiative of initiativesIssues) {
            issuesTree.appendChild(treeRoot, initiative);
            const children = await fetchChildren_1.default(userConfig, initiative.key, cacheDir, cache);
            for (const l1child of children.filter((ic) => ic.fields[userConfig.jira.fields.parentInitiative] === initiative.key)) {
                issuesTree.appendChild(initiative, l1child);
                for (const l2child of children.filter((ic) => ic.fields[userConfig.jira.fields.parentEpic] === l1child.key)) {
                    issuesTree.appendChild(l1child, l2child);
                }
            }
        }
        cli_ux_1.default.action.start('Preparing metrics for all initiatives in the tree ');
        // Update the tree with completion and week by week metrics
        prepareInitiativesData_1.default(issuesTree, treeRoot, 0, closedIssues, emptyCalendar, userConfig);
        cli_ux_1.default.action.stop(' done');
        cli_ux_1.default.action.start('Creating an export payload');
        const rawInitiativesTree = treeUtils_1.exportTree(issuesTree, treeRoot);
        cli_ux_1.default.action.stop(' done');
        cli_ux_1.default.action.start('Preparing orphan issues');
        //Get an array of all orphaned issues
        const orphanIssues = prepareOrphanIssues_1.default(issuesTree, treeRoot, closedIssues);
        //Add orphaned issues to a weekly calendar
        const orphanWeekly = emptyCalendar.map((week) => {
            const weekIssues = orphanIssues.filter((issue) => week.weekStart === issue.weekStart);
            return Object.assign(Object.assign({}, week), { list: weekIssues, issues: {
                    count: weekIssues.length,
                }, points: {
                    count: weekIssues
                        .map((issue) => issue.points)
                        .reduce((acc, points) => acc + points, 0),
                } });
        });
        cli_ux_1.default.action.stop(' done');
        cli_ux_1.default.action.start('Forecasting future completion');
        const emptyRoadmap = getEmptyRoadmap_1.default(emptyCalendar[emptyCalendar.length - 1], 
        // tslint:disable-next-line: strict-type-predicates
        userConfig.roadmap.forecastWeeks !== undefined
            ? userConfig.roadmap.forecastWeeks
            : 26);
        const futureCompletion = crunchRoadmap_1.default(emptyRoadmap, velocityTeamCache, rawInitiativesTree);
        cli_ux_1.default.action.stop(' done');
        const rawInitiativesArtifact = {
            updatedAt: new Date().toJSON(),
            host: userConfig.jira.host,
            calendar: {
                completed: emptyCalendar,
                roadmap: emptyRoadmap,
            },
            initiatives: rawInitiativesTree,
            orphanIssues: orphanWeekly,
            futureCompletion: futureCompletion,
        };
        cli_ux_1.default.action.start('Optimizing payload for frontend');
        const slimInitiativesArtifact = optimizePayload_1.default(rawInitiativesArtifact);
        cli_ux_1.default.action.stop(' done');
        // Export data to file
        const initiativesRawFileStream = fs.createWriteStream(path.join(cacheDir, 'initiatives-artifacts-raw.json'), { flags: 'w' });
        initiativesRawFileStream.write(JSON.stringify(rawInitiativesArtifact));
        initiativesRawFileStream.end();
        const initiativesSlimFileStream = fs.createWriteStream(path.join(cacheDir, 'initiatives-artifacts.json'), { flags: 'w' });
        initiativesSlimFileStream.write(JSON.stringify(slimInitiativesArtifact));
        initiativesSlimFileStream.end();
    }
}
exports.default = Roadmap;
Roadmap.description = 'Builds a roadmap from a set of issues';
Roadmap.flags = Object.assign(Object.assign({}, base_1.default.flags), { help: command_1.flags.help({ char: 'h' }), type: command_1.flags.string({
        char: 't',
        description: 'Use issues of points for metrics',
        options: ['issues', 'points'],
        default: 'points',
    }), cache: command_1.flags.boolean({
        char: 'c',
        description: 'Use cached version of the child issues (mostly useful for dev)',
        default: false,
    }) });
