"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const cli_ux_1 = require("cli-ux");
const fs = require("fs");
const path = require("path");
const base_1 = require("../base");
const fetchCompleted_1 = require("../utils/data/fetchCompleted");
const searchIssues_1 = require("../utils/jira/searchIssues");
const teamUtils_1 = require("../utils/misc/teamUtils");
const getDailyHealthMsg_1 = require("../utils/slack/getDailyHealthMsg");
const sendSlackMsg_1 = require("../utils/slack/sendSlackMsg");
const initCalendar_1 = require("../utils/velocity/initCalendar");
const insertClosed_1 = require("../utils/velocity/insertClosed");
const insertDailyVelocity_1 = require("../utils/velocity/insertDailyVelocity");
const insertForecast_1 = require("../utils/velocity/insertForecast");
const insertHealth_1 = require("../utils/velocity/insertHealth");
const insertOpen_1 = require("../utils/velocity/insertOpen");
const insertWeeklyVelocity_1 = require("../utils/velocity/insertWeeklyVelocity");
const jiraUtils_1 = require("../utils/misc/jiraUtils");
class Velocity extends base_1.default {
    constructor() {
        super(...arguments);
        /*
          Fetch open issues from Jira
        */
        this.fetchOpenIssues = async (userConfig, teamName) => {
            const teamConfig = userConfig.teams.find(t => t.name === teamName);
            if (teamConfig !== undefined) {
                const openIssues = [];
                cli_ux_1.default.action.start('Fetching open issues for team: ' +
                    teamName +
                    ' using JQL: ' +
                    teamConfig.jqlRemaining +
                    ' ');
                const issuesJira = await searchIssues_1.default(userConfig.jira, teamConfig.jqlRemaining, 'labels,' + userConfig.jira.fields.points);
                for (const issue of issuesJira) {
                    openIssues.push(jiraUtils_1.cleanIssue(issue));
                }
                cli_ux_1.default.action.stop(' done');
                return issuesJira;
            }
            return [];
        };
    }
    async run() {
        const { flags } = this.parse(Velocity);
        const userConfig = this.userConfig;
        // eslint-disable-next-line
        let { type, dryrun } = flags;
        for (const team of userConfig.teams) {
            const closedIssues = await fetchCompleted_1.default(userConfig, this.config.configDir + '/cache/', team.name);
            console.log('Fetched ' + closedIssues.length + ' completed issues');
            const emptyCalendar = initCalendar_1.default(team.jqlHistory);
            const calendarWithClosed = await insertClosed_1.default(emptyCalendar, userConfig.jira.fields.points, closedIssues);
            const openIssues = await this.fetchOpenIssues(userConfig, team.name);
            const calendarWithOpen = insertOpen_1.default(calendarWithClosed, openIssues, userConfig.jira.fields.points);
            const calendarVelocity = Object.assign(Object.assign({}, calendarWithOpen), { days: insertDailyVelocity_1.default(calendarWithOpen), weeks: insertWeeklyVelocity_1.default(calendarWithOpen) });
            const calendarWithForecast = insertForecast_1.default(calendarVelocity);
            const calendarWithHealth = Object.assign(Object.assign({}, insertHealth_1.default(calendarWithForecast)), { updatedAt: new Date().toJSON(), host: userConfig.jira.host, jqlCompletion: team.jqlCompletion // Adding JQL completion query
             });
            // Prune issues list from the object
            const trimmedPayload = Object.assign(Object.assign({}, calendarWithHealth), { days: calendarWithHealth.days.map((d) => {
                    if (d.completion.list !== undefined) {
                        delete d.completion.list;
                    }
                    if (d.scopeChangeCompletion.list !== undefined) {
                        delete d.scopeChangeCompletion.list;
                    }
                    return d;
                }), weeks: calendarWithHealth.weeks.map((w) => {
                    if (w.completion.list !== undefined) {
                        delete w.completion.list;
                    }
                    if (w.scopeChangeCompletion.list !== undefined) {
                        delete w.scopeChangeCompletion.list;
                    }
                    return w;
                }), open: (o) => {
                    if (o.list !== undefined) {
                        delete o.list;
                    }
                    return o;
                } });
            const slackMsg = getDailyHealthMsg_1.default(trimmedPayload, type, userConfig, team.name);
            this.log(slackMsg);
            if (!dryrun) {
                cli_ux_1.default.action.start('Sending message to Slack');
                sendSlackMsg_1.default(team.slack.token, team.slack.channel, slackMsg);
                cli_ux_1.default.action.stop(' done');
            }
            const cacheDir = this.config.configDir + '/cache/';
            const issueFileStream = fs.createWriteStream(path.join(cacheDir, 'velocity-artifacts-' + teamUtils_1.getTeamId(team.name) + '.json'), { flags: 'w' });
            issueFileStream.write(JSON.stringify(trimmedPayload));
            issueFileStream.end();
        }
    }
}
exports.default = Velocity;
Velocity.description = 'Builds velocity stats by day and week';
Velocity.flags = Object.assign(Object.assign({}, base_1.default.flags), { help: command_1.flags.help({ char: 'h' }), type: command_1.flags.string({
        char: 't',
        description: 'Send slack update using issues or points',
        options: ['issues', 'points'],
        default: 'points'
    }), dryrun: command_1.flags.boolean({
        char: 'd',
        default: false,
        description: 'Dry-Run, do not send slack message'
    }) });
exports.formatDate = (dateString) => {
    const day = new Date(dateString);
    day.setUTCHours(12);
    day.setUTCMinutes(0);
    day.setUTCSeconds(0);
    return day;
};
